/*
[INFO] 

NAME=CUI Manager (With Params)
TYPE=Installation Extension (With Params)
VERSION=1.0.0
AUTHOR=MastaMan
DEV=3DGROUND
SITE=http://3dground.net
DATE=03 May, 2024
THANKS=
REQ=
DESC=This script is an extension to the installer, which will create menu items in the interface due to the 3D Max 2025 update

[SCRIPT]

*/

struct icuiWrapper (
	selfPath = getFileNamePath (getThisScriptFileName()),
	iniFile = selfPath + @"\install.ini",
	iniSettingsFile = selfPath + @"\settings.ini",
	macro = "",
	cat = "",
	txt = "",
	help = "",
	icons = false,
	currVer = "",
	startupFile = "",
	insertBefore = "Select &Similar",
	separatorIdStart = "E8D87BEA-5FD3-4270-8FE4-596407C076E6",
	separatorIdEnd = "E8D87BEB-5FD3-4270-8FE4-596407C076E6",
	scriptData = #(),	
	errors = #(
		"500. Error when using CUI, script data not received!",
		"404. Can't write file, CUI script!"
	),
		
	fn getIni p sec: "INSTALL" t: #string s: false = (	
		local f = iniFile
		if(s == true) do f = iniSettingsFile
		
		local i = getIniSetting f sec p
		if (p == "") do i = getIniSetting f sec
		
		case t of (
			#bool: return (if(i == "" or i == "false") then false else true)
			#int: return (try(i as integer) catch(0))
			default: return i
		)	
	),
	
	fn safeCopyFile f1 f2 = (
		try(setFileAttribute f2 #readOnly false) catch()
		deleteFile f2
		return copyFile f1 f2
	),
	
	fn safeDeleteFile f = (
		if (not doesFileExist f) do return false
		try(setFileAttribute f #readOnly false) catch()
		return deleteFile f
	),
	
	fn writeFile f s = (
		try (
			f = createFile f --mode: "wt"
			
			format (s as string) to:f
			
			flush f
			close f
		) catch (
			print (errors[2])
		)
	),
	
	fn getScriptsData  = (
		local quads = getIni "" sec: "QUAD"
		local o = #()
		struct tmp (name, macro, cat)
		
		for s in quads do (
			n = getFileNameFile s
			m = "QuadMenu_" + (substituteString n " " "")
			
			append o (tmp name: n macro: m cat: cat)
		)
		
		return o
	),
	
	fn createStartupCuiFile = (
		if (startupFile == "") do return (print errors[1])
		local s = "try(fileIn (getDir(#temp) + \"\\" + macro + "\\CUI.ms\"))catch(print (\"Error while load " + txt + " CUI!\"))"
		writeFile startupFile s
	),
	
	fn removeStartupCuiFile = (
		if (startupFile == "") do return (print errors[1])
		safeDeleteFile startupFile
	),
	
	fn findQuadMenuItem find = (
		local quadMenuMgr = maxops.getICUiQuadMenuMgr()
		local id = undefined
		local menu = undefined
		local item = undefined
		local title = undefined
		local findId = toLower find
		
		struct tmp (id, menu, item, title)
		
		for quadContext in quadMenuMgr.contexts do (	
			for quadMenu in quadContext.quadMenus do (		
				for subMenu in quadMenu.menus do (
					for i in 1 to subMenu.menuItems.count do (
						title = try (subMenu.menuItems[i].title) catch (undefined)
						id = toLower (subMenu.menuItems[i].id)
						
						if (title == find or id == findId) do (
							menu = subMenu
							item = subMenu.menuItems[i]
							return tmp id: id menu: menu item: item title: title
						)						
					) 				
				)	
			)
		)
		
		return tmp id: id menu: menu item: item title: title
	),
	
	fn removeSeparators = (
		local separators = #(separatorIdStart, separatorIdEnd)
		for s in separators do (
			local m = findQuadMenuItem s
			if (m.id != undefined and m.menu != undefined) do m.menu.deleteItem m.id
		)		
	),
	
	fn createQuadMenuItem title macro cat before separator: #none = (
		local o = findQuadMenuItem title
		
		if (o.id != undefined and o.menu != undefined) do (
			o.menu.DeleteItem o.id
		)
		
		local m = findQuadMenuItem before
		
		if (m.id != undefined) do (
			if (separator == #start) do m.menu.CreateSeparator separatorIdStart beforeId: m.id
			m.menu.CreateAction (genGUID()) 647394 (macro + "`[" + cat + "]") title: title beforeId: m.id
			if (separator == #end) do m.menu.CreateSeparator separatorIdEnd beforeId: m.id
		)
		
		local quadMenuMgr = maxops.getICUiQuadMenuMgr()
		quadMenuMgr.setCurrentConfiguration (quadMenuMgr.GetDefaultConfiguration())
	),
	
	fn install = (
		if (scriptData.count == 0) do return (print errors[1])		
		createStartupCuiFile()
		
		removeSeparators()
		
		for i in 1 to scriptData.count do (
			local d = scriptData[i]
			local s = #none
			if (i == 1) do s = #start
			if (i == scriptData.count) do s = #end	
			
			createQuadMenuItem d.name d.macro d.cat insertBefore separator: s
		)			
	),
	
	fn uninstall = (
		if (scriptData.count == 0) do return (print errors[1])
		
		for d in scriptData do (
			local o = findQuadMenuItem d.name
			if (o.id != undefined and o.menu != undefined) do (
				o.menu.DeleteItem o.id
			)
		)
		
		removeSeparators()
		removeStartupCuiFile()
	),
	
	on create do (
		macro = getIni "MACRO"
		cat = getIni "CAT"
		txt = getIni "TXT"
		help = getIni "HELP"
		icons = getIni "ICONS" t: #bool
		currVer = getIni "CURR_VER" sec: "UPDATE"
		
		scriptData = getScriptsData()
		
		startupFile = (getDir #userStartupScripts) + @"\" + macro + "_CUI.ms"
	)
)

(icuiWrapper()).install()
--(icuiWrapper()).uninstall()
